/*  Programa INSERE.
 *  Programa recebe pela linha de comando  nome de 2 arquivos de texto, e
 *  5 inteiros Exemplo:
 * 
 *      insere img03.pgm img01.pgm 200 10 100 180 150
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define TAM_FRASE  255
#define RESULTADO  "img-res.pgm"

typedef char Pixel;

/* Prototipos */
Pixel **lerImagem(FILE *fp, int *largura, int *altura, int *maxval);
void gravaImagem(Pixel **imagem, int largura, int altura, int maxval);


/* Programa Principal */
main (int argc, char * argv[]) 
{
  Pixel **imgOrg1, **imgOrg2;
  int i, j, k, tam, largura, altura, maxval,
      linRes, linBloco, colBloco, altBloco, largBloco;
  FILE *fp1, *fp2;

  /* verifica se todos os argumentos da linha de comando foram dados */
  /* Por que 8 e nao 7? */
  if (argc != 8) {
    printf("Entre com 7 args: arq1 arq2 linha x y alt larg \n");
    exit(1);
  }

  /* Abre os arquivos para leitura */
  fp1 = fopen (argv[1], "r");
  fp2 = fopen (argv[2], "r");
  if (fp1 == NULL || fp2 == NULL) {
    printf("Erro ao abrir arquivos %s ou %s\n", argv[1], argv[2]);
    exit(1);
  }

  /* Coordenadas de insercao e do bloco */    
    linRes = atoi (argv[3]);
    linBloco = atoi (argv[4]);
    colBloco = atoi (argv[5]);
    altBloco = atoi (argv[6]);
    largBloco = atoi (argv[7]);

    /* Leitura das imagens em memoria */
    /* 2a. imagem */
    imgOrg2 = lerImagem(fp2, &largura, &altura, &maxval);
    /* 1a. imagem */
    /* largura, altura e maxval contem as dimensoes da 1a. imagem */
    imgOrg1 = lerImagem(fp1, &largura, &altura, &maxval);

  /* Insere bloco do 2o para o 1o */
  /* Realoca imgOrg1 para caber a insercao do bloco.
   * novo tamanho eh Coluna * (Linhas + altBloco)
   */
  imgOrg1 = (Pixel **) realloc (imgOrg1, (altura + altBloco) * sizeof(Pixel *));
  
  /* Copia bloco de imgOrg1 que comeca em linRes para regiao
   * abaixo do trecho onde serah escrito o bloco de imgOrg2
   */
  for (i = altura - 1; i > linRes; i--)
  {
    imgOrg1[altBloco + i] = imgOrg1[i];
    
    if (i < linRes+altBloco)
    {
      imgOrg1[i] = (Pixel *) calloc (largura, sizeof(Pixel));
      /* OPCIONAL: "pinta" a area nova de BRANCO */
      memset(imgOrg1[i], maxval, largura);
    }
  }
  
  /* Copia bloco de imgOrg2 para imgOrg1, comecando na linha linRes */
  for (i = 0, j = 0; i < altBloco; i++, j++)
  {
    tam = largBloco > largura ? largura : largBloco;
    for (k = 0; k < tam; k++)
      imgOrg1[linRes+i][k] =imgOrg2[linBloco+j][colBloco+k];
  }
  altura += altBloco;

  /* Salva imagem */
  gravaImagem(imgOrg1, largura, altura, maxval);
  
  /* fecha arquivos */
  fclose(fp1);
  fclose(fp2);
  
} /*FIM PROGRAMA */


Pixel **lerImagem(FILE *fp, int *largura, int *altura, int *maxval)
{
  Pixel **img;
  char buffer[TAM_FRASE + 1] = "";
  int i;

  /* Leitura da Identificacao */
  fgets(buffer, 5, fp);

  /* Leitura das dimensoes da figura */
  fgets(buffer, TAM_FRASE, fp);
  sscanf(buffer, "%d %d", largura, altura);

  /*Leitura do valor maximo para BRANCO */
  fgets(buffer, TAM_FRASE, fp);
  sscanf(buffer, "%d", maxval);
  
  /* aloca memoria para ler 2o. arquivo */  
  img = (Pixel **) calloc (*altura, sizeof(Pixel *));
  if (img == NULL) {
    printf ("ERRO DE ALOCACAO DE TEXTO1\n");
    exit(1);
  }

  /* Leitura do 2o. arquivo, usando vetor de ponteiros */
  for (i = 0; i < *altura; i++) 
  {
    img[i] = (Pixel *) calloc (*largura, sizeof(Pixel));
    fread(img[i], sizeof(Pixel), *largura, fp);
  }

  return (img);
}



void gravaImagem(Pixel **imagem, int largura, int altura, int maxval)
{
    FILE *fp;
    int i;

    /* Cria arquivo para gravar resultado */
    fp = fopen (RESULTADO, "w");
    if (fp == NULL) {
      printf("Erro ao criar arquivo %s\n", RESULTADO);
      exit(1);
    }  

    /* Salva resultado em arquivo */
    /* Salva Identificacao */
    fputs("P5\n", fp);

    /* Salva as dimensoes da nova figura */
    fprintf(fp, "%d %d\n", largura, altura);

    /*Salva do valor maximo para BRANCO */
    fprintf(fp, "%d\n", maxval);

    /* forcar a gravacao dos dados em disco, antes de haver um fclose() */
    fflush(fp);
  
    for (i = 0; i < altura; i++) { 
      fwrite(imagem[i], sizeof(Pixel), largura, fp);
      /* forcar a gravacao dos dados em disco, antes de haver um fclose() */
      fflush(fp);
    }

    fclose(fp);
    
}


